@@ -24,15 +24,34 @@
 #include "gstjackaudioclient.h"
 #include "gstjack.h"
 
-#include <gst/glib-compat-private.h>
-
 GST_DEBUG_CATEGORY_STATIC (gst_jack_audio_client_debug);
 #define GST_CAT_DEFAULT gst_jack_audio_client_debug
 
+/* List of threads who are trying to establish an initial server connection.
+ * Poor man's TLS. We don't really want to waste a TLS ID for this, and the
+ * error log callback code path should not be performance sensitive anyway. */
+G_LOCK_DEFINE_STATIC (startups_lock);
+static GList *startups;
+
 static void
 jack_log_error (const gchar * msg)
 {
-  GST_ERROR ("%s", msg);
+  gboolean starting_up;
+
+  G_LOCK (startups_lock);
+  starting_up = startups && g_list_find (startups, g_thread_self ()) != NULL;
+  G_UNLOCK (startups_lock);
+
+  /* Not being able to connect to a JACK server is completely normal in case
+   * jackaudiosink or jackaudiosrc are autoplugged, so we don't want to spew
+   * ERROR log messages in that case. On Linux this is rarely noticable because
+   * pulseaudiosink has a higher rank and is chosen first and alsa comes
+   * alphabetically before jack. */
+  if (starting_up) {
+    GST_WARNING ("%s", msg);
+  } else {
+    GST_ERROR ("%s", msg);
+  }
 }
 
 static void
@@ -47,8 +66,8 @@ gst_jack_audio_client_init (void)
   GST_DEBUG_CATEGORY_INIT (gst_jack_audio_client_debug, "jackclient", 0,
       "jackclient helpers");
 
-  jack_set_error_function (jack_log_error);
-  jack_set_info_function (jack_info_error);
+  gst_jack_set_error_function (jack_log_error);
+  gst_jack_set_info_function (jack_info_error);
 }
 
 /* a list of global connections indexed by id and server. */
@@ -123,7 +142,7 @@ jack_process_cb (jack_nframes_t nframes, void *arg)
   GstJackAudioConnection *conn = (GstJackAudioConnection *) arg;
   GList *walk;
   int res = 0;
-  jack_transport_state_t ts = jack_transport_query (conn->client, NULL);
+  jack_transport_state_t ts = gst_jack_transport_query (conn->client, NULL);
 
   if (ts != conn->cur_ts) {
     conn->cur_ts = ts;
@@ -295,7 +314,7 @@ gst_jack_audio_make_connection (const gchar * id, const gchar * server,
     options |= JackServerName;
   /* open the client */
   if (jclient == NULL)
-    jclient = jack_client_open (id, options, status, server);
+    jclient = gst_jack_client_open (id, options, status, server);
   if (jclient == NULL)
     goto could_not_open;
 
@@ -314,15 +333,15 @@ gst_jack_audio_make_connection (const gchar * id, const gchar * server,
   conn->transport_state = GST_STATE_VOID_PENDING;
 
   /* set our callbacks  */
-  jack_set_process_callback (jclient, jack_process_cb, conn);
+  gst_jack_set_process_callback (jclient, jack_process_cb, conn);
   /* these callbacks cause us to error */
-  jack_set_buffer_size_callback (jclient, jack_buffer_size_cb, conn);
-  jack_set_sample_rate_callback (jclient, jack_sample_rate_cb, conn);
-  jack_on_shutdown (jclient, jack_shutdown_cb, conn);
+  gst_jack_set_buffer_size_callback (jclient, jack_buffer_size_cb, conn);
+  gst_jack_set_sample_rate_callback (jclient, jack_sample_rate_cb, conn);
+  gst_jack_on_shutdown (jclient, jack_shutdown_cb, conn);
 
   /* all callbacks are set, activate the client */
   GST_INFO ("activate jack_client %p", jclient);
-  if ((res = jack_activate (jclient)))
+  if ((res = gst_jack_activate (jclient)))
     goto could_not_activate;
 
   GST_DEBUG ("opened connection %p", conn);
@@ -409,21 +428,21 @@ gst_jack_audio_unref_connection (GstJackAudioConnection * conn)
     /* don't use conn->lock here. two reasons:
      *
      *  1) its not necessary: jack_deactivate() will not return until the JACK thread
-     *      associated with this connection is cleaned up by a thread join, hence 
+     *      associated with this connection is cleaned up by a thread join, hence
      *      no more callbacks can occur or be in progress.
      *
      * 2) it would deadlock anyway, because jack_deactivate() will sleep
-     *      waiting for the JACK thread, and can thus cause deadlock in 
+     *      waiting for the JACK thread, and can thus cause deadlock in
      *      jack_process_cb()
      */
     GST_INFO ("deactivate jack_client %p", conn->client);
-    if ((res = jack_deactivate (conn->client))) {
+    if ((res = gst_jack_deactivate (conn->client))) {
       /* we only warn, this means the server is probably shut down and the client
        * is gone anyway. */
       GST_WARNING ("Could not deactivate Jack client (%d)", res);
     }
     /* close connection */
-    if ((res = jack_client_close (conn->client))) {
+    if ((res = gst_jack_client_close (conn->client))) {
       /* we assume the client is gone. */
       GST_WARNING ("close failed (%d)", res);
     }
@@ -494,7 +513,7 @@ gst_jack_audio_connection_remove_client (GstJackAudioConnection * conn,
  * Get the jack client connection for @id and @server. Connections to the same
  * @id and @server will receive the same physical Jack client connection and
  * will therefore be scheduled in the same process callback.
- * 
+ *
  * Returns: a #GstJackAudioClient.
  */
 GstJackAudioClient *
@@ -504,16 +523,25 @@ gst_jack_audio_client_new (const gchar * id, const gchar * server,
     JackBufferSizeCallback buffer_size, JackSampleRateCallback sample_rate,
     gpointer user_data, jack_status_t * status)
 {
-  GstJackAudioClient *client;
+  GstJackAudioClient *client = NULL;
   GstJackAudioConnection *conn;
 
   g_return_val_if_fail (id != NULL, NULL);
   g_return_val_if_fail (status != NULL, NULL);
 
+  /* So the error log callback knows that we're doing an initial connection
+   * attempt */
+  G_LOCK (startups_lock);
+  startups = g_list_prepend (startups, g_thread_self ());
+  G_UNLOCK (startups_lock);
+
   /* first get a connection for the id/server pair */
   conn = gst_jack_audio_get_connection (id, server, jclient, status);
-  if (conn == NULL)
+
+  if (conn == NULL) {
+    GST_DEBUG ("Could not get server connection (%d)", *status);
     goto no_connection;
+  }
 
   GST_INFO ("new client %s", id);
 
@@ -532,14 +560,14 @@ gst_jack_audio_client_new (const gchar * id, const gchar * server,
   /* add the client to the connection */
   gst_jack_audio_connection_add_client (conn, client);
 
-  return client;
-
-  /* ERRORS */
 no_connection:
-  {
-    GST_DEBUG ("Could not get server connection (%d)", *status);
-    return NULL;
-  }
+
+  /* done connecting */
+  G_LOCK (startups_lock);
+  startups = g_list_remove (startups, g_thread_self ());
+  G_UNLOCK (startups_lock);
+
+  return client;
 }
 
 /**
@@ -581,7 +609,7 @@ gst_jack_audio_client_get_client (GstJackAudioClient * client)
 {
   g_return_val_if_fail (client != NULL, NULL);
 
-  /* no lock needed, the connection and the client does not change 
+  /* no lock needed, the connection and the client does not change
    * once the client is created. */
   return client->conn->client;
 }
@@ -591,7 +619,7 @@ gst_jack_audio_client_get_client (GstJackAudioClient * client)
  * @client: a #GstJackAudioClient
  * @active: new mode for the client
  *
- * Activate or deactive @client. When a client is activated it will receive
+ * Activate or deactivate @client. When a client is activated it will receive
  * callbacks when data should be processed.
  *
  * Returns: 0 if all ok.
@@ -635,3 +663,54 @@ gst_jack_audio_client_get_transport_state (GstJackAudioClient * client)
   client->conn->transport_state = GST_STATE_VOID_PENDING;
   return state;
 }
+
+/**
+ * gst_jack_audio_client_get_port_names_from_string:
+ * @jclient: a jack_client_t handle
+ * @port_names: comma-separated jack port name(s)
+ * @port_flags: JackPortFlags
+ *
+ * Returns: a newly-allocated %NULL-terminated array of strings or %NULL
+ *   if @port_names contains invalid port name. Use g_strfreev() to free it.
+ */
+gchar **
+gst_jack_audio_client_get_port_names_from_string (jack_client_t * jclient,
+    const gchar * port_names, gint port_flags)
+{
+  gchar **p = NULL;
+  guint i, len;
+
+  g_return_val_if_fail (jclient != NULL, NULL);
+
+  if (!port_names)
+    return NULL;
+
+  p = g_strsplit (port_names, ",", 0);
+  len = g_strv_length (p);
+
+  if (len < 1)
+    goto invalid;
+
+  for (i = 0; i < len; i++) {
+    jack_port_t *port = gst_jack_port_by_name (jclient, p[i]);
+    int flags;
+
+    if (!port) {
+      GST_WARNING ("Couldn't get jack port by name %s", p[i]);
+      goto invalid;
+    }
+
+    flags = gst_jack_port_flags (port);
+    if ((flags & port_flags) != port_flags) {
+      GST_WARNING ("Port flags 0x%x doesn't match expected flags 0x%x",
+          flags, port_flags);
+      goto invalid;
+    }
+  }
+
+  return p;
+
+invalid:
+  g_strfreev (p);
+  return NULL;
+}
