@@ -52,6 +52,7 @@ typedef struct _GstMatroskaDemux {
 
   /* pads */
   GstClock                *clock;
+  gboolean                 have_nonintraonly_v_streams;
   guint                    num_v_streams;
   guint                    num_a_streams;
   guint                    num_t_streams;
@@ -67,8 +68,10 @@ typedef struct _GstMatroskaDemux {
   gboolean                 seek_first;
 
   /* did we parse cues/tracks/segmentinfo already? */
-  gboolean                 tracks_parsed;
   GList                   *seek_parsed;
+  /* Offset of the last Tracks element parsed,
+   * to avoid reparsing or recreating tracks unecessarily */
+  guint64                 tracks_ebml_offset;
 
   /* cluster positions (optional) */
   GArray                  *clusters;
@@ -84,10 +87,12 @@ typedef struct _GstMatroskaDemux {
   /* some state saving */
   GstClockTime             cluster_time;
   guint64                  cluster_offset;
+  guint64                  cluster_prevsize;       /* 0 if unknown */
   guint64                  first_cluster_offset;
   guint64                  next_cluster_offset;
   GstClockTime             requested_seek_time;
   guint64                  seek_offset;
+  GstClockTime             audio_lead_in_ts;
 
   /* alternative duration; optionally obtained from last cluster */
   guint64                  last_cluster_offset;
@@ -98,13 +103,30 @@ typedef struct _GstMatroskaDemux {
   gboolean                 building_index;
   guint64                  index_offset;
   GstEvent                *seek_event;
+  GstEvent                *deferred_seek_event;
+  GstPad                  *deferred_seek_pad;
   gboolean                 need_segment;
   guint32                  segment_seqnum;
+  /* If TRUE, matroskamux received upstream newsegment in TIME format
+   * which likely means that upstream is driving the pipeline (such as
+   * adaptive demuxers) */
+  gboolean                 upstream_format_is_time;
 
   /* reverse playback */
   GArray                  *seek_index;
   gint                     seek_entry;
 
+  gboolean                 seen_cluster_prevsize;  /* We track this because the
+                                                    * first cluster won't have
+                                                    * this set, so we can't just
+                                                    * check cluster_prevsize to
+                                                    * determine if it's there
+                                                    * or not. We assume if one
+                                                    * cluster has it, all but
+                                                    * the first will have it. */
+
+  guint                    max_backtrack_distance; /* in seconds (0 = don't backtrack) */
+
   /* gap handling */
   guint64                  max_gap_time;
 
@@ -119,8 +141,6 @@ typedef struct _GstMatroskaDemuxClass {
   GstElementClass parent;
 } GstMatroskaDemuxClass;
 
-gboolean gst_matroska_demux_plugin_init (GstPlugin *plugin);
-
 G_END_DECLS
 
 #endif /* __GST_MATROSKA_DEMUX_H__ */
