@@ -26,6 +26,7 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/audio/audio.h>
 
+#include "gstrtpelements.h"
 #include "gstrtpceltpay.h"
 #include "gstrtputils.h"
 
@@ -66,6 +67,8 @@ static GstFlowReturn gst_rtp_celt_pay_handle_buffer (GstRTPBasePayload *
 
 #define gst_rtp_celt_pay_parent_class parent_class
 G_DEFINE_TYPE (GstRtpCELTPay, gst_rtp_celt_pay, GST_TYPE_RTP_BASE_PAYLOAD);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpceltpay, "rtpceltpay",
+    GST_RANK_SECONDARY, GST_TYPE_RTP_CELT_PAY, rtp_element_init (plugin));
 
 static void
 gst_rtp_celt_pay_class_init (GstRtpCELTPayClass * klass)
@@ -495,10 +498,3 @@ gst_rtp_celt_pay_change_state (GstElement * element, GstStateChange transition)
   }
   return ret;
 }
-
-gboolean
-gst_rtp_celt_pay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpceltpay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_CELT_PAY);
-}
