@@ -50,6 +50,7 @@ struct _GstRtpVorbisPay
 
   /* queues of buffers along with some stats. */
   GstBuffer    *packet;
+  GList        *packet_buffers;
   guint         payload_pos;
   guint         payload_left;
   guint32       payload_ident;
@@ -77,8 +78,6 @@ struct _GstRtpVorbisPayClass
 
 GType gst_rtp_vorbis_pay_get_type (void);
 
-gboolean gst_rtp_vorbis_pay_plugin_init (GstPlugin * plugin);
-
 G_END_DECLS
 
 #endif /* __GST_RTP_VORBIS_PAY_H__ */
