@@ -26,6 +26,7 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include <gst/video/video.h>
 
+#include "gstrtpelements.h"
 #include "gstrtpvrawpay.h"
 #include "gstrtputils.h"
 
@@ -84,9 +85,12 @@ static void gst_rtp_vraw_pay_get_property (GObject * object, guint prop_id,
 static void gst_rtp_vraw_pay_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 
-G_DEFINE_TYPE (GstRtpVRawPay, gst_rtp_vraw_pay, GST_TYPE_RTP_BASE_PAYLOAD)
+G_DEFINE_TYPE (GstRtpVRawPay, gst_rtp_vraw_pay, GST_TYPE_RTP_BASE_PAYLOAD);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (rtpvrawpay, "rtpvrawpay",
+    GST_RANK_SECONDARY, GST_TYPE_RTP_VRAW_PAY, rtp_element_init (plugin));
 
-     static void gst_rtp_vraw_pay_class_init (GstRtpVRawPayClass * klass)
+static void
+gst_rtp_vraw_pay_class_init (GstRtpVRawPayClass * klass)
 {
   GstRTPBasePayloadClass *gstrtpbasepayload_class;
   GstElementClass *gstelement_class;
@@ -281,6 +285,7 @@ gst_rtp_vraw_pay_handle_buffer (GstRTPBasePayload * payload, GstBuffer * buffer)
   gboolean use_buffer_lists;
   GstBufferList *list = NULL;
   GstRTPBuffer rtp = { NULL, };
+  gboolean discont;
 
   rtpvrawpay = GST_RTP_VRAW_PAY (payload);
 
@@ -289,6 +294,8 @@ gst_rtp_vraw_pay_handle_buffer (GstRTPBasePayload * payload, GstBuffer * buffer)
     return GST_FLOW_ERROR;
   }
 
+  discont = GST_BUFFER_IS_DISCONT (buffer);
+
   GST_LOG_OBJECT (rtpvrawpay, "new frame of %" G_GSIZE_FORMAT " bytes",
       gst_buffer_get_size (buffer));
 
@@ -325,7 +332,8 @@ gst_rtp_vraw_pay_handle_buffer (GstRTPBasePayload * payload, GstBuffer * buffer)
   buffers_per_list = packlines_per_list * packets_per_packline;
   buffers_per_list = GST_ROUND_UP_8 (buffers_per_list);
 
-  use_buffer_lists = (rtpvrawpay->chunks_per_frame < (height / yinc));
+  use_buffer_lists = buffers_per_list > 1 &&
+      (rtpvrawpay->chunks_per_frame < (height / yinc));
 
   fields = 1 + interlaced;
 
@@ -348,7 +356,13 @@ gst_rtp_vraw_pay_handle_buffer (GstRTPBasePayload * payload, GstBuffer * buffer)
 
       /* get the max allowed payload length size, we try to fill the complete MTU */
       left = gst_rtp_buffer_calc_payload_len (mtu, 0, 0);
-      out = gst_rtp_buffer_new_allocate (left, 0, 0);
+      out = gst_rtp_base_payload_allocate_output_buffer (payload, left, 0, 0);
+
+      if (discont) {
+        GST_BUFFER_FLAG_SET (out, GST_BUFFER_FLAG_DISCONT);
+        /* Only the first outputted buffer has the DISCONT flag */
+        discont = FALSE;
+      }
 
       if (field == 0) {
         GST_BUFFER_PTS (out) = GST_BUFFER_PTS (buffer);
@@ -552,6 +566,7 @@ gst_rtp_vraw_pay_handle_buffer (GstRTPBasePayload * payload, GstBuffer * buffer)
       if (line >= height) {
         GST_LOG_OBJECT (rtpvrawpay, "field/frame complete, set marker");
         gst_rtp_buffer_set_marker (&rtp, TRUE);
+        GST_BUFFER_FLAG_SET (out, GST_BUFFER_FLAG_MARKER);
         complete = TRUE;
       }
       gst_rtp_buffer_unmap (&rtp);
@@ -645,10 +660,3 @@ gst_rtp_vraw_pay_get_property (GObject * object, guint prop_id,
       break;
   }
 }
-
-gboolean
-gst_rtp_vraw_pay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpvrawpay",
-      GST_RANK_SECONDARY, GST_TYPE_RTP_VRAW_PAY);
-}
