@@ -74,10 +74,14 @@ struct _GstRtpBin {
   guint32         max_dropout_time;
   guint32         max_misorder_time;
   gboolean        rfc7273_sync;
+  gboolean        add_reference_timestamp_meta;
   guint           max_streams;
   guint64         max_ts_offset_adjustment;
   gint64          max_ts_offset;
   gboolean        max_ts_offset_is_set;
+  guint64         min_ts_offset;
+  gboolean        min_ts_offset_is_set;
+  guint           ts_offset_smoothing_factor;
 
   /* a list of session */
   GSList         *sessions;
@@ -88,6 +92,16 @@ struct _GstRtpBin {
   /* the default SDES items for sessions */
   GstStructure   *sdes;
 
+  /* the default FEC decoder factories for sessions */
+  GstStructure   *fec_decoders;
+
+  /* the default FEC encoder factories for sessions */
+  GstStructure   *fec_encoders;
+
+  gboolean       update_ntp64_header_ext;
+
+  gboolean       timeout_inactive_sources;
+
   /*< private >*/
   GstRtpBinPrivate *priv;
 };
@@ -111,6 +125,7 @@ struct _GstRtpBinClass {
   RTPSession* (*get_internal_session) (GstRtpBin *rtpbin, guint session);
   GstElement* (*get_storage)          (GstRtpBin *rtpbin, guint session);
   GObject*    (*get_internal_storage) (GstRtpBin *rtpbin, guint session);
+  void        (*clear_ssrc)           (GstRtpBin *rtpbin, guint session, guint32 ssrc);
 
   /* session manager signals */
   void     (*on_new_ssrc)       (GstRtpBin *rtpbin, guint session, guint32 ssrc);
@@ -143,4 +158,6 @@ struct _GstRtpBinClass {
 
 GType gst_rtp_bin_get_type (void);
 
+GST_ELEMENT_REGISTER_DECLARE (rtpbin);
+
 #endif /* __GST_RTP_BIN_H__ */
