@@ -23,6 +23,7 @@
 #include <gst/gst.h>
 
 #include "rtpsource.h"
+#include "rtptwcc.h"
 
 typedef struct _RTPSession RTPSession;
 typedef struct _RTPSessionClass RTPSessionClass;
@@ -96,16 +97,16 @@ typedef GstFlowReturn (*RTPSessionSendRTCP) (RTPSession *sess, RTPSource *src, G
 typedef GstFlowReturn (*RTPSessionSyncRTCP) (RTPSession *sess, GstBuffer *buffer, gpointer user_data);
 
 /**
- * RTPSessionClockRate:
+ * RTPSessionCaps:
  * @sess: an #RTPSession
  * @payload: the payload
  * @user_data: user data specified when registering
  *
- * This callback will be called when @sess needs the clock-rate of @payload.
+ * This callback will be called when @sess needs the caps of @payload.
  *
- * Returns: the clock-rate of @pt.
+ * Returns: the caps of @pt.
  */
-typedef gint (*RTPSessionClockRate) (RTPSession *sess, guint8 payload, gpointer user_data);
+typedef GstCaps * (*RTPSessionCaps) (RTPSession *sess, guint8 payload, gpointer user_data);
 
 /**
  * RTPSessionReconsider:
@@ -156,6 +157,15 @@ typedef GstClockTime (*RTPSessionRequestTime) (RTPSession *sess,
 typedef void (*RTPSessionNotifyNACK) (RTPSession *sess,
     guint16 seqnum, guint16 blp, guint32 ssrc, gpointer user_data);
 
+/**
+ * RTPSessionNotifyTWCC:
+ * @user_data: user data specified when registering
+ *
+ * Notifies of Transport-wide congestion control packets and stats.
+ */
+typedef void (*RTPSessionNotifyTWCC) (RTPSession *sess,
+    GstStructure * twcc_packets, GstStructure * twcc_stats, gpointer user_data);
+
 /**
  * RTPSessionReconfigure:
  * @sess: an #RTPSession
@@ -186,6 +196,7 @@ typedef void (*RTPSessionNotifyEarlyRTCP) (RTPSession *sess,
  * @RTPSessionRequestKeyUnit: callback for requesting a new key unit
  * @RTPSessionRequestTime: callback for requesting the current time
  * @RTPSessionNotifyNACK: callback for notifying NACK
+ * @RTPSessionNotifyTWCC: callback for notifying TWCC
  * @RTPSessionReconfigure: callback for requesting reconfiguration
  * @RTPSessionNotifyEarlyRTCP: callback for notifying early RTCP
  *
@@ -198,11 +209,12 @@ typedef struct {
   RTPSessionSendRTP     send_rtp;
   RTPSessionSyncRTCP    sync_rtcp;
   RTPSessionSendRTCP    send_rtcp;
-  RTPSessionClockRate   clock_rate;
+  RTPSessionCaps        caps;
   RTPSessionReconsider  reconsider;
   RTPSessionRequestKeyUnit request_key_unit;
   RTPSessionRequestTime request_time;
   RTPSessionNotifyNACK  notify_nack;
+  RTPSessionNotifyTWCC  notify_twcc;
   RTPSessionReconfigure reconfigure;
   RTPSessionNotifyEarlyRTCP notify_early_rtcp;
 } RTPSessionCallbacks;
@@ -264,10 +276,10 @@ struct _RTPSession {
   GstClockTime  last_rtcp_interval;   /* T_rr */
   GstClockTime  start_time;
   gboolean      first_rtcp;
-  gboolean      allow_early;
 
   GstClockTime  next_early_rtcp_time;
 
+  gboolean      sr_req_pending;
   gboolean      scheduled_bye;
 
   RTPSessionCallbacks   callbacks;
@@ -275,11 +287,12 @@ struct _RTPSession {
   gpointer              send_rtp_user_data;
   gpointer              send_rtcp_user_data;
   gpointer              sync_rtcp_user_data;
-  gpointer              clock_rate_user_data;
+  gpointer              caps_user_data;
   gpointer              reconsider_user_data;
   gpointer              request_key_unit_user_data;
   gpointer              request_time_user_data;
   gpointer              notify_nack_user_data;
+  gpointer              notify_twcc_user_data;
   gpointer              reconfigure_user_data;
   gpointer              notify_early_rtcp_user_data;
 
@@ -295,6 +308,17 @@ struct _RTPSession {
   GList         *conflicting_addresses;
 
   gboolean timestamp_sender_reports;
+
+  /* RFC6051 64-bit NTP header extension */
+  guint8 send_ntp64_ext_id;
+
+  gboolean update_ntp64_header_ext;
+
+  gboolean timeout_inactive_sources;
+
+  /* Transport-wide cc-extension */
+  RTPTWCCManager *twcc;
+  RTPTWCCStats *twcc_stats;
 };
 
 /**
@@ -354,8 +378,8 @@ void            rtp_session_set_send_rtcp_callback   (RTPSession * sess,
 void            rtp_session_set_sync_rtcp_callback   (RTPSession * sess,
                                                     RTPSessionSyncRTCP callback,
                                                     gpointer user_data);
-void            rtp_session_set_clock_rate_callback   (RTPSession * sess,
-                                                    RTPSessionClockRate callback,
+void            rtp_session_set_caps_callback        (RTPSession * sess,
+                                                    RTPSessionCaps callback,
                                                     gpointer user_data);
 void            rtp_session_set_reconsider_callback (RTPSession * sess,
                                                     RTPSessionReconsider callback,
@@ -393,7 +417,8 @@ GstFlowReturn   rtp_session_process_rtcp           (RTPSession *sess, GstBuffer
 /* processing packets for sending */
 void            rtp_session_update_send_caps       (RTPSession *sess, GstCaps *caps);
 GstFlowReturn   rtp_session_send_rtp               (RTPSession *sess, gpointer data, gboolean is_list,
-                                                    GstClockTime current_time, GstClockTime running_time);
+                                                    GstClockTime current_time, GstClockTime running_time,
+                                                    guint64 ntpnstime);
 
 /* scheduling bye */
 void            rtp_session_mark_all_bye           (RTPSession *sess, const gchar *reason);
@@ -404,7 +429,7 @@ GstClockTime    rtp_session_next_timeout           (RTPSession *sess, GstClockTi
 GstFlowReturn   rtp_session_on_timeout             (RTPSession *sess, GstClockTime current_time,
                                                     guint64 ntpnstime, GstClockTime running_time);
 
-/* request the transmittion of an early RTCP packet */
+/* request the transmission of an early RTCP packet */
 gboolean        rtp_session_request_early_rtcp     (RTPSession * sess, GstClockTime current_time,
                                                     GstClockTime max_delay);
 
@@ -418,5 +443,7 @@ gboolean        rtp_session_request_nack           (RTPSession * sess,
                                                     guint16 seqnum,
                                                     GstClockTime max_delay);
 
+void            rtp_session_update_recv_caps_structure (RTPSession * sess, const GstStructure * s);
+
 
 #endif /* __RTP_SESSION_H__ */
