@@ -438,68 +438,46 @@ gst_v4l2_src_parse_fixed_struct (GstStructure * s,
     gst_structure_get_fraction (s, "framerate", fps_n, fps_d);
 }
 
-/* TODO Consider framerate */
 static gint
 gst_v4l2src_fixed_caps_compare (GstCaps * caps_a, GstCaps * caps_b,
     struct PreferredCapsInfo *pref)
 {
   GstStructure *a, *b;
-  gint aw = G_MAXINT, ah = G_MAXINT, ad = G_MAXINT;
-  gint bw = G_MAXINT, bh = G_MAXINT, bd = G_MAXINT;
-  gint ret;
+  gint aw = G_MAXINT, ah = G_MAXINT;
+  gint bw = G_MAXINT, bh = G_MAXINT;
+  gint a_fps_n = G_MAXINT, a_fps_d = 1;
+  gint b_fps_n = G_MAXINT, b_fps_d = 1;
+  gint a_distance, b_distance;
+  gint ret = 0;
 
   a = gst_caps_get_structure (caps_a, 0);
   b = gst_caps_get_structure (caps_b, 0);
 
-  gst_v4l2_src_parse_fixed_struct (a, &aw, &ah, NULL, NULL);
-  gst_v4l2_src_parse_fixed_struct (b, &bw, &bh, NULL, NULL);
+  gst_v4l2_src_parse_fixed_struct (a, &aw, &ah, &a_fps_n, &a_fps_d);
+  gst_v4l2_src_parse_fixed_struct (b, &bw, &bh, &b_fps_n, &b_fps_d);
 
-  /* When both are smaller then pref, just append to the end */
-  if ((bw < pref->width || bh < pref->height)
-      && (aw < pref->width || ah < pref->height)) {
-    ret = 1;
-    goto done;
-  }
+  // Sort first the one with closest framerate to preference. Note that any
+  // framerate lower then 1 frame per second will be considered the same. In
+  // practice this should be fine considering that these framerate only exists
+  // for still picture, in which case the resolution is most likely the key.
+  a_distance = ABS ((a_fps_n / a_fps_d) - (pref->fps_n / pref->fps_d));
+  b_distance = ABS ((b_fps_n / b_fps_d) - (pref->fps_n / pref->fps_d));
+  if (a_distance != b_distance)
+    return a_distance - b_distance;
 
-  /* If a is smaller then pref and not b, then a goes after b */
-  if (aw < pref->width || ah < pref->height) {
-    ret = 1;
-    goto done;
-  }
+  // If same framerate, sort first the one with closest resolution to preference
+  a_distance = ABS (aw * ah - pref->width * pref->height);
+  b_distance = ABS (bw * bh - pref->width * pref->height);
 
-  /* If b is smaller then pref and not a, then a goes before b */
-  if (bw < pref->width || bh < pref->height) {
-    ret = -1;
-    goto done;
-  }
-
-  /* Both are larger or equal to the preference, prefer the smallest */
-  ad = MAX (1, aw - pref->width) * MAX (1, ah - pref->height);
-  bd = MAX (1, bw - pref->width) * MAX (1, bh - pref->height);
-
-  /* Adjust slightly in case width/height matched the preference */
-  if (aw == pref->width)
-    ad -= 1;
-
-  if (ah == pref->height)
-    ad -= 1;
-
-  if (bw == pref->width)
-    bd -= 1;
-
-  if (bh == pref->height)
-    bd -= 1;
-
-  /* If the choices are equivalent, maintain the order */
-  if (ad == bd)
+  /* If the distance are equivalent, maintain the order */
+  if (a_distance == b_distance)
     ret = 1;
   else
-    ret = ad - bd;
+    ret = a_distance - b_distance;
+
+  GST_TRACE ("Placing %" GST_PTR_FORMAT " %s %" GST_PTR_FORMAT,
+      caps_a, ret > 0 ? "after" : "before", caps_b);
 
-done:
-  GST_TRACE ("Placing %ix%i (%s) %s %ix%i (%s)", aw, ah,
-      gst_structure_get_string (a, "format"), ret > 0 ? "after" : "before", bw,
-      bh, gst_structure_get_string (b, "format"));
   return ret;
 }
 
@@ -585,6 +563,14 @@ gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps,
    * enumerate the possibilities */
   caps = gst_caps_normalize (caps);
 
+  /* try hard to avoid TRY_FMT since some UVC camera just crash when this
+   * is called at run-time. */
+  if (gst_v4l2_object_caps_is_subset (obj, caps)) {
+    fcaps = gst_v4l2_object_get_current_caps (obj);
+    GST_DEBUG_OBJECT (basesrc, "reuse current caps %" GST_PTR_FORMAT, fcaps);
+    goto out;
+  }
+
   for (i = 0; i < gst_caps_get_size (caps); ++i) {
     gst_v4l2_clear_error (&error);
     if (fcaps)
@@ -592,14 +578,6 @@ gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps,
 
     fcaps = gst_caps_copy_nth (caps, i);
 
-    /* try hard to avoid TRY_FMT since some UVC camera just crash when this
-     * is called at run-time. */
-    if (gst_v4l2_object_caps_is_subset (obj, fcaps)) {
-      gst_caps_unref (fcaps);
-      fcaps = gst_v4l2_object_get_current_caps (obj);
-      break;
-    }
-
     /* Just check if the format is acceptable, once we know
      * no buffers should be outstanding we try S_FMT.
      *
@@ -633,6 +611,7 @@ gst_v4l2src_fixate (GstBaseSrc * basesrc, GstCaps * caps,
     return NULL;
   }
 
+out:
   gst_caps_unref (caps);
 
   GST_DEBUG_OBJECT (basesrc, "fixated caps %" GST_PTR_FORMAT, fcaps);
@@ -1148,6 +1127,23 @@ gst_v4l2src_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
+static gboolean
+gst_v4l2src_handle_resolution_change (GstV4l2Src * v4l2src)
+{
+  GST_INFO_OBJECT (v4l2src, "Resolution change detected.");
+
+  /* It is required to always cycle through streamoff, we also need to
+   * streamoff in order to allow locking a new DV_TIMING which will
+   * influence the output of TRY_FMT */
+  gst_v4l2src_stop (GST_BASE_SRC (v4l2src));
+
+  /* Force renegotiation */
+  v4l2src->renegotiation_adjust = v4l2src->offset + 1;
+  v4l2src->pending_set_fmt = TRUE;
+
+  return gst_base_src_negotiate (GST_BASE_SRC (v4l2src));
+}
+
 static GstFlowReturn
 gst_v4l2src_create (GstPushSrc * src, GstBuffer ** buf)
 {
@@ -1166,18 +1162,7 @@ gst_v4l2src_create (GstPushSrc * src, GstBuffer ** buf)
 
     if (G_UNLIKELY (ret != GST_FLOW_OK)) {
       if (ret == GST_V4L2_FLOW_RESOLUTION_CHANGE) {
-        GST_INFO_OBJECT (v4l2src, "Resolution change detected.");
-
-        /* It is required to always cycle through streamoff, we also need to
-         * streamoff in order to allow locking a new DV_TIMING which will
-         * influence the output of TRY_FMT */
-        gst_v4l2src_stop (GST_BASE_SRC (src));
-
-        /* Force renegotiation */
-        v4l2src->renegotiation_adjust = v4l2src->offset + 1;
-        v4l2src->pending_set_fmt = TRUE;
-
-        if (!gst_base_src_negotiate (GST_BASE_SRC (src))) {
+        if (!gst_v4l2src_handle_resolution_change (v4l2src)) {
           ret = GST_FLOW_NOT_NEGOTIATED;
           goto error;
         }
@@ -1193,6 +1178,13 @@ gst_v4l2src_create (GstPushSrc * src, GstBuffer ** buf)
       ret = gst_v4l2_buffer_pool_process (obj_pool, buf, NULL);
       if (obj_pool)
         gst_object_unref (obj_pool);
+
+      if (G_UNLIKELY (ret == GST_V4L2_FLOW_RESOLUTION_CHANGE)) {
+        if (!gst_v4l2src_handle_resolution_change (v4l2src)) {
+          ret = GST_FLOW_NOT_NEGOTIATED;
+          goto error;
+        }
+      }
     }
 
   } while (ret == GST_V4L2_FLOW_CORRUPTED_BUFFER ||
