@@ -433,6 +433,43 @@ GST_START_TEST (test_splitmuxsink)
 
 GST_END_TEST;
 
+GST_START_TEST (test_splitmuxsink_clean_failure)
+{
+  GstMessage *msg;
+  GstElement *pipeline;
+  GstElement *sink, *fakesink;
+
+  /* This pipeline has a small time cutoff - it should start a new file
+   * every GOP, ie 1 second */
+  pipeline =
+      gst_parse_launch
+      ("videotestsrc horizontal-speed=2 is-live=true ! video/x-raw,width=80,height=64,framerate=5/1 ! videoconvert !"
+      " queue ! theoraenc keyframe-force=5 ! splitmuxsink name=splitsink "
+      " max-size-time=1000000 max-size-bytes=1000000 muxer=oggmux", NULL);
+  fail_if (pipeline == NULL);
+  sink = gst_bin_get_by_name (GST_BIN (pipeline), "splitsink");
+  fail_if (sink == NULL);
+
+  fakesink = gst_element_factory_make ("fakesink", "fakesink-fail");
+  fail_if (fakesink == NULL);
+
+  /* Trigger an error on READY->PAUSED */
+  g_object_set (fakesink, "state-error", 2, NULL);
+  g_object_set (sink, "sink", fakesink, NULL);
+  gst_object_unref (sink);
+
+  msg = run_pipeline (pipeline);
+
+  fail_unless (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ERROR);
+  gst_message_unref (msg);
+
+  fail_unless (gst_element_set_state (pipeline,
+          GST_STATE_NULL) == GST_STATE_CHANGE_SUCCESS);
+  gst_object_unref (pipeline);
+}
+
+GST_END_TEST;
+
 GST_START_TEST (test_splitmuxsink_multivid)
 {
   GstMessage *msg;
@@ -559,7 +596,7 @@ GST_START_TEST (test_splitmuxsink_reuse_simple)
   GstPad *pad;
 
   sink = gst_element_factory_make ("splitmuxsink", NULL);
-  pad = gst_element_get_request_pad (sink, "video");
+  pad = gst_element_request_pad_simple (sink, "video");
   fail_unless (pad != NULL);
   g_object_set (sink, "location", "/dev/null", NULL);
 
@@ -598,13 +635,13 @@ GST_START_TEST (test_splitmuxsink_muxer_pad_map)
   g_object_set (sink, "muxer", muxer, "muxer-pad-map", pad_map, NULL);
   gst_structure_free (pad_map);
 
-  pad1 = gst_element_get_request_pad (sink, "video");
+  pad1 = gst_element_request_pad_simple (sink, "video");
   fail_unless (g_str_equal ("video", GST_PAD_NAME (pad1)));
   muxpad = gst_element_get_static_pad (muxer, "video_100");
   fail_unless (muxpad != NULL);
   gst_object_unref (muxpad);
 
-  pad2 = gst_element_get_request_pad (sink, "audio_0");
+  pad2 = gst_element_request_pad_simple (sink, "audio_0");
   fail_unless (g_str_equal ("audio_0", GST_PAD_NAME (pad2)));
   muxpad = gst_element_get_static_pad (muxer, "audio_101");
   fail_unless (muxpad != NULL);
@@ -626,6 +663,55 @@ GST_START_TEST (test_splitmuxsink_muxer_pad_map)
 
 GST_END_TEST;
 
+static void
+run_eos_pipeline (guint num_video_buf, guint num_audio_buf,
+    gboolean configure_audio)
+{
+  GstMessage *msg;
+  GstElement *pipeline;
+  gchar *dest_pattern;
+  gchar *pipeline_str;
+  gchar *audio_branch = NULL;
+
+  dest_pattern = g_build_filename (tmpdir, "out%05d.mp4", NULL);
+
+  if (configure_audio) {
+    audio_branch = g_strdup_printf ("audiotestsrc num-buffers=%d ! "
+        "splitsink.audio_0", num_audio_buf);
+  }
+
+  pipeline_str = g_strdup_printf ("splitmuxsink name=splitsink location=%s "
+      "muxer-factory=qtmux videotestsrc num-buffers=%d ! jpegenc ! splitsink. "
+      "%s", dest_pattern, num_video_buf, audio_branch ? audio_branch : "");
+  pipeline = gst_parse_launch (pipeline_str, NULL);
+  g_free (dest_pattern);
+  g_free (audio_branch);
+  g_free (pipeline_str);
+
+  fail_if (pipeline == NULL);
+
+  msg = run_pipeline (pipeline);
+
+  if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ERROR)
+    dump_error (msg);
+  fail_unless (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_EOS);
+  gst_message_unref (msg);
+
+  gst_object_unref (pipeline);
+}
+
+GST_START_TEST (test_splitmuxsink_eos_without_buffer)
+{
+  /* below pipelines will create non-playable files but at least we should not
+   * crash */
+  run_eos_pipeline (0, 0, FALSE);
+  run_eos_pipeline (0, 0, TRUE);
+  run_eos_pipeline (1, 0, TRUE);
+  run_eos_pipeline (0, 1, TRUE);
+}
+
+GST_END_TEST;
+
 static GstPadProbeReturn
 count_upstrea_fku (GstPad * pad, GstPadProbeInfo * info,
     guint * upstream_fku_count)
@@ -807,6 +893,7 @@ splitmuxsink_suite (void)
     tcase_add_checked_fixture (tc_chain, tempdir_setup, tempdir_cleanup);
 
     tcase_add_test (tc_chain, test_splitmuxsink);
+    tcase_add_test (tc_chain, test_splitmuxsink_clean_failure);
 
     if (have_matroska && have_vorbis) {
       tcase_add_checked_fixture (tc_chain_complex, tempdir_setup,
@@ -825,6 +912,7 @@ splitmuxsink_suite (void)
     tcase_add_checked_fixture (tc_chain_mp4_jpeg, tempdir_setup,
         tempdir_cleanup);
     tcase_add_test (tc_chain_mp4_jpeg, test_splitmuxsink_muxer_pad_map);
+    tcase_add_test (tc_chain_mp4_jpeg, test_splitmuxsink_eos_without_buffer);
   } else {
     GST_INFO ("Skipping tests, missing plugins: jpegenc or mp4mux");
   }
